Attribute VB_Name = "MSTR"
'===============================================================================================
'This code is distributed "as is".
'No warranty of any kind is expressed or implied. You use it at your own risk.
'I disclaim all liabilities for consequential data loss, damages, loss of profits or any other
'kind of loss while using or misusing it.
'
'-----------------------------------------------------------------------------------------------
'Please ask for my approval at ssorin@gmail.com before distributing or posting
'this code to a site other than http://www.microstrategy101.com or http://microstrategyblog.com/
'===============================================================================================



'In this module we will have a look at three ways of finding an object
'1. FindObjectByName, where the name and type are specified
'2. FindObjectByID, where the ID and type are specified
'3. FindObjectbyID (overloaded function) where only the ID is specified

Function FindObjectByName(Name As String, DSSType As EnumDSSObjectType) As IDSSObjectInfo
        
        '==========================
        'This is the old school way
        '==========================

        
        Dim objsearch As IDSSSearch
        Dim result As IDSSFolder

        Set objsearch = objSession.ObjectSource.NewObject(DssTypeSearch)

        objsearch.NamePattern = Name
        objsearch.Flags = DssSearchNameWildCard
        objsearch.Types.Add (DSSType)


        Set result = objsearch.Execute

        'There may be more objects with the same name and type (though not recommended).
        'We shall keep the first found, if any.
        If result.Count > 0 Then
            Set FindObjectByName = result.Item(1)
        Else: Set FindObjectByName = Nothing
        End If





End Function

Function FindObjectByID2(ByVal ID As String, ByVal DSSType As EnumDSSObjectType) As IDSSObjectInfo
    
        '==============================
        'This is another old school way
        '==============================

        'FindObject is a native MicroStrategy function that searches an object based on its ID and type
        Set FindObjectByID2 = objSession.ObjectSource.FindObject(ID, DSSType)


End Function

Function FindObjectByID1(ByVal ID As String) As IDSSObjectInfo

        '======================
        'This is the real deal.
        '======================
        
        'Though slower, this function will eventually find any object, based on its ID alone.


        'The trick is to keep ignoring errors.
        On Error Resume Next
        'Otherwise, whenever FindObject attempts to search for the attribute with ID "xyz" but "xyz" is actually a metric you will get a show stopper error

        Dim result As IDSSObjectInfo
        

        'Here be some damn clever code
        'EnumDSSObjectType can also be expressed as a number. So far the greatest such number is 58
        For i = 1 To 58
            Set result = objSession.ObjectSource.FindObject(ID, i)
            
            'If an object is found, then stop processing the rest of the loop
            If Not result Is Nothing Then Exit For
        Next


        Set FindObjectByID1 = result


End Function
